@@ -237,6 +237,7 @@ puts {
 #  ifdef HAVE_STDLIB_H
 #    include <stdlib.h>
 #  endif
+#  include <tcl.h>
 
 #  ifndef LOADED_CVFS_COMMON
 #    define LOADED_CVFS_COMMON 1
@@ -286,12 +287,12 @@ static unsigned long cvfs_hash(const unsigned char *path) {
         return(h);
 }
 
-static int cvfs_decrypt(unsigned char *out, const unsigned char *in, unsigned long in_out_length, struct cvfs_key *key) {
+static int cvfs_decrypt(unsigned char *out, const unsigned char *in, unsigned long offset, unsigned long in_out_length, struct cvfs_key *key) {
 	unsigned long i;
 	unsigned char in_ch, out_ch;
 	int ch_idx;
 
-	for (i = 0; i < in_out_length; i++) {
+	for (i = offset; i < in_out_length; i++) {
 		in_ch = in[i];
 
 		ch_idx = (in_ch + (i / key->typedata.rotate_subst.rotate_length)) % 256;
@@ -512,10 +513,10 @@ if {$obsfucate} {
 	puts "\tunsigned char *new_data, *old_data;"
 	puts "\tint decrypt_ret, free_old_data;"
 	puts ""
-	puts "\tnew_data = malloc(finfo->size);"
-	puts "\tdecrypt_ret = cvfs_decrypt(new_data, finfo->data, finfo->size, &key);"
+	puts "\tnew_data = (void *) Tcl_Alloc(finfo->size);"
+	puts "\tdecrypt_ret = cvfs_decrypt(new_data, finfo->data, 0, finfo->size, &key);"
 	puts "\tif (decrypt_ret != 0) {"
-	puts "\t\tfree(new_data);"
+	puts "\t\tTcl_Free((void *) new_data);"
 	puts ""
 	puts "\t\treturn;"
 	puts "\t}"
@@ -528,7 +529,7 @@ if {$obsfucate} {
 	puts "\tfinfo->type = CVFS_FILETYPE_FILE;"
 	puts ""
 	puts "\tif (free_old_data) {"
-	puts "\t\tfree(old_data);"
+	puts "\t\tTcl_Free((void *) old_data);"
 	puts "\t}"
 	puts "\treturn;"
 	puts "}"
@@ -562,6 +563,14 @@ puts "int Cvfs_data_${hashkey}_Init(Tcl_Interp *interp) {"
 puts "\tTcl_Command tclCreatComm_ret;"
 puts "\tint tclPkgProv_ret;"
 puts ""
+puts "#ifdef USE_TCL_STUBS"
+puts "\tif (!Tcl_InitStubs(interp, TCL_VERSION, 0)) {"
+puts "#else"
+puts "\tif (!Tcl_PkgRequire(interp, \"Tcl\", TCL_VERSION, 0)) {"
+puts "#endif /* USE_TCL_STUBS */"
+puts "\t\treturn(TCL_ERROR);"
+puts "\t}"
+puts ""
 puts "\ttclCreatComm_ret = Tcl_CreateObjCommand(interp, \"::vfs::cvfs::data::${hashkey}::getMetadata\", getMetadata, NULL, NULL);"
 puts "\tif (!tclCreatComm_ret) {"
 puts "\t\treturn(TCL_ERROR);"
